package com.io.yutian.auchestshop.listener;

import com.io.yutian.auchestshop.AuChestShop;
import com.io.yutian.auchestshop.gui.CreateShopGui;
import com.io.yutian.auchestshop.shop.Shop;
import com.io.yutian.auchestshop.util.ExpiringSet;
import com.io.yutian.aulib.listener.IListener;
import com.io.yutian.aulib.nbt.NBTItem;
import com.io.yutian.aulib.nbt.NBTString;
import it.unimi.dsi.fastutil.Pair;
import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.block.data.type.Chest;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.block.Action;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.block.BlockPlaceEvent;
import org.bukkit.event.player.AsyncPlayerChatEvent;
import org.bukkit.event.player.PlayerAnimationEvent;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.inventory.EquipmentSlot;
import org.bukkit.inventory.ItemStack;
import org.bukkit.plugin.Plugin;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

public class ShopListener extends IListener {

    private final ExpiringSet<UUID> adventureWorkaround = new ExpiringSet<>(1, TimeUnit.SECONDS);
    private final Set<Location> cachedLocations = new HashSet<>();

    public ShopListener(Plugin plugin) {
        super(plugin);
    }

    @EventHandler(priority = EventPriority.LOWEST, ignoreCancelled = true)
    public void onChat(AsyncPlayerChatEvent event) {
        Player player = event.getPlayer();
        UUID uuid = player.getUniqueId();
        if (!AuChestShop.getInteractiveManager().containsKey(uuid)) {
            return;
        }
        AuChestShop.getInteractiveManager().handler(uuid, event.getMessage());
        event.setCancelled(true);
    }

    @EventHandler(priority = EventPriority.LOW, ignoreCancelled = true)
    public void onBlockPlace(BlockPlaceEvent event) {
        Player player = event.getPlayer();
        Block block = event.getBlock();
        ItemStack itemStack = event.getItemInHand();
        NBTItem nbtItem = new NBTItem(itemStack);
        if (nbtItem.has("item_type", NBTString.TYPE_ID)) {
            String itemType = (String) nbtItem.get("item_type").getValue();
            if (itemType.equals("chest_shop")) {
                CreateShopGui createShopGui = new CreateShopGui(player, block, itemStack, ()->{
                    cachedLocations.remove(block.getLocation());
                });
                createShopGui.open();
                cachedLocations.add(block.getLocation());
            }
        }
    }

    @EventHandler(priority = EventPriority.LOW, ignoreCancelled = true)
    public void onBlockBreak(BlockBreakEvent event) {
        Player player = event.getPlayer();
        Block block = event.getBlock();
        if (cachedLocations.contains(block.getLocation())) {
            event.setCancelled(true);
        }
    }

    @EventHandler(priority = EventPriority.LOW, ignoreCancelled = true)
    public void onAdventureClick(PlayerAnimationEvent event) {
        if (event.getPlayer().getGameMode() != GameMode.ADVENTURE) {
            return;
        }
        if (adventureWorkaround.contains(event.getPlayer().getUniqueId())) {
            return;
        }
        adventureWorkaround.add(event.getPlayer().getUniqueId());
        Block focused = event.getPlayer().getTargetBlockExact(5);
        if (focused != null) {
            PlayerInteractEvent interactEvent
                    = new PlayerInteractEvent(event.getPlayer(),
                    focused.getType() == Material.AIR ? Action.LEFT_CLICK_AIR : Action.LEFT_CLICK_BLOCK,
                    event.getPlayer().getInventory().getItemInMainHand(),
                    focused,
                    event.getPlayer().getFacing().getOppositeFace());
            onPlayerInteract(interactEvent);
        }
    }

    @EventHandler(priority = EventPriority.LOW, ignoreCancelled = true)
    public void onPlayerInteract(PlayerInteractEvent event) {
        if (event.getHand() != EquipmentSlot.HAND) {
            return;
        }
        Player player = event.getPlayer();
        if (player.getGameMode() == GameMode.ADVENTURE) {
            if (!adventureWorkaround.contains(player.getUniqueId())) {
                return;
            }
            adventureWorkaround.add(player.getUniqueId());
        }
        Action action = event.getAction();
        Block clickedBlock = event.getClickedBlock();
        if (clickedBlock == null) {
            return;
        }
        if (cachedLocations.contains(clickedBlock.getLocation())) {
            event.setCancelled(true);
            return;
        }
        Pair<Shop, ClickType> shopSearched = searchShop(player, clickedBlock);
        if (shopSearched.key() == null || shopSearched.value() == ClickType.AIR) {
            return;
        }
        Shop shop = shopSearched.key();
        ClickType clickType = shopSearched.value();
        System.out.println(shop);
    }

    private Pair<Shop, ClickType> searchShop(Player player, Block block) {
        if (block == null || block.getType() == Material.AIR) {
            return Pair.of(null, ClickType.AIR);
        }
        Shop shop = AuChestShop.getShopManager().getShop(block.getLocation());
        Material material = block.getType();
        if (shop == null) {
            if (material == Material.CHEST || material == Material.TRAPPED_CHEST) {
                Chest chest = (Chest) block.getState().getBlockData();
                Chest.ChestDoubleBlockHalfType halfType = chest.getDoubleBlockHalf();
                if (halfType == Chest.ChestDoubleBlockHalfType.LOWER) {
                    return Pair.of(null, ClickType.AIR);
                } else if (halfType == Chest.ChestDoubleBlockHalfType.UPPER) {
                    Block attached = block.getRelative(BlockFace.DOWN, 1);
                    shop = AuChestShop.getShopManager().getShop(attached.getLocation());
                    return Pair.of(shop, ClickType.UPPER);
                }
            } else {
                return Pair.of(null, ClickType.AIR);
            }
        }
        return Pair.of(shop, ClickType.LOWER);
    }

    enum ClickType {
        UPPER,
        LOWER,
        AIR;
    }

}
